/*
** Job Arranger for ZABBIX
** Copyright (C) 2012 FitechForce, Inc. All Rights Reserved.
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
**/

/*
** $Date:: 2012-11-26 13:15:38 +0900 #$
** $Revision: 2725 $
** $Author: ossinfra@FITECHLABS.CO.JP $
**/

#include "common.h"
#include "comms.h"
#include "log.h"
#include "sysinfo.h"

#include "jacommon.h"
#include "jajobobject.h"
#include "jatelegram.h"
#include "jatcp.h"

#ifndef ZBX_SOCKLEN_T
#define ZBX_SOCKLEN_T socklen_t
#endif

#ifdef HAVE_IPV6
#define ZBX_SOCKADDR struct sockaddr_storage
#else
#define ZBX_SOCKADDR struct sockaddr_in
#endif

/******************************************************************************
 *                                                                            *
 * Function:                                                                  *
 *                                                                            *
 * Purpose:                                                                   *
 *                                                                            *
 * Parameters:                                                                *
 *                                                                            *
 * Return value:                                                              *
 *                                                                            *
 * Comments:                                                                  *
 *                                                                            *
 ******************************************************************************/
int ja_tcp_accept(zbx_sock_t * s)
{
    ZBX_SOCKADDR serv_addr;
    ZBX_SOCKLEN_T nlen;
    const char *__function_name = "ja_tcp_accept";

    zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __function_name);

    zbx_sock_close(s->socket);
    nlen = sizeof(serv_addr);

    if (ZBX_SOCK_ERROR ==
        (s->socket =
         (ZBX_SOCKET) accept(s->sockets[0], (struct sockaddr *) &serv_addr,
                             &nlen))) {
        zabbix_log(LOG_LEVEL_WARNING, "accept() failed: %s",
                   strerror_from_system(zbx_sock_last_error()));

        return FAIL;
    }
    s->accepted = 1;

    return SUCCEED;
}

/******************************************************************************
 *                                                                            *
 * Function:                                                                  *
 *                                                                            *
 * Purpose:                                                                   *
 *                                                                            *
 * Parameters:                                                                *
 *                                                                            *
 * Return value:                                                              *
 *                                                                            *
 * Comments:                                                                  *
 *                                                                            *
 ******************************************************************************/
int ja_tcp_send(char *data)
{
    int ret;
    char *buf;
    zbx_sock_t sock;
    ja_job_object job;
    const char *__function_name = "ja_tcp_send";

    zabbix_log(LOG_LEVEL_DEBUG, "In %s() sending [%s]", __function_name,
               data);

    ret = zbx_tcp_connect(&sock, CONFIG_SOURCE_IP,
                          CONFIG_HOSTS_ALLOWED, CONFIG_SERVER_PORT,
                          CONFIG_TIMEOUT);
    if (ret == FAIL) {
        zabbix_log(LOG_LEVEL_WARNING, "Send value error: [connect] %s",
                   zbx_tcp_strerror());

        return FAIL;
    }

    ret = zbx_tcp_send(&sock, data);
    if (ret == FAIL) {
        zabbix_log(LOG_LEVEL_WARNING, "Send value error: [send] %s",
                   zbx_tcp_strerror());
        goto error;

    }

    ret = zbx_tcp_recv(&sock, &buf);
    if (ret == FAIL) {
        zabbix_log(LOG_LEVEL_WARNING, "Send value error: [recv] %s",
                   zbx_tcp_strerror());
        goto error;

    }
    zabbix_log(LOG_LEVEL_DEBUG, "Sent [%s]", buf);

    ja_job_object_init(&job);
    ret = ja_telegram_from_jobresult_res(buf, &job);
    if (ret == FAIL) {
        zabbix_log(LOG_LEVEL_ERR, "In %s() %s", __function_name,
                   job.message);
    }

    if (job.result == JA_JOBRESULT_RES_SUCCEED)
        ret = SUCCEED;
    else
        ret = FAIL;

  error:
    zbx_tcp_close(&sock);
    return ret;

}
