/*
** Job Arranger for ZABBIX
** Copyright (C) 2012 FitechForce, Inc. All Rights Reserved.
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
**/

/*
** $Date:: 2012-11-26 13:15:38 +0900 #$
** $Revision: 2725 $
** $Author: ossinfra@FITECHLABS.CO.JP $
**/

#include <json/json.h>
#include "common.h"
#include "log.h"

#include "jacommon.h"
#include "jastr.h"
#include "jajobobject.h"
#include "jatelegram.h"

/******************************************************************************
 *                                                                            *
 * Function:                                                                  *
 *                                                                            *
 * Purpose:                                                                   *
 *                                                                            *
 * Parameters:                                                                *
 *                                                                            *
 * Return value:                                                              *
 *                                                                            *
 * Comments:                                                                  *
 *                                                                            *
 ******************************************************************************/
int ja_telegram_from_head(json_object * json, ja_job_object * job)
{
    json_object *jp;
    char *telegram;
    const char *__function_name = "ja_telegram_from_head";

    if (json == NULL || job == NULL)
        return FAIL;

    telegram = (char *) json_object_get_string(json);
    zabbix_log(LOG_LEVEL_DEBUG, "In %s() telegram: %s", __function_name,
               telegram);

    if (json_object_get_type(json) != json_type_object) {
        zbx_snprintf(job->message, sizeof(job->message),
                     "[%s] is not an object", telegram);
        job->result = JA_JOBRUN_RES_FAIL;
        return 2;
    }
    // KIND
    jp = json_object_object_get(json, JA_PROTO_TAG_KIND);
    if (jp != NULL) {
        zbx_snprintf(job->kind, sizeof(job->kind),
                     "%s", json_object_get_string(jp));
    } else {
        zbx_snprintf(job->message, sizeof(job->message),
                     "Can't get the tag [%s] from json data [%s]",
                     JA_PROTO_TAG_KIND, telegram);
        return 2;
    }

    // SERVERID
    jp = json_object_object_get(json, JA_PROTO_TAG_SERVERID);
    if (jp != NULL) {
        zbx_snprintf(job->serverid, sizeof(job->serverid),
                     "%s", json_object_get_string(jp));
    } else {
        zbx_snprintf(job->message, sizeof(job->message),
                     "Can't get the tag [%s] from json data [%s]",
                     JA_PROTO_TAG_SERVERID, telegram);
        return 2;
    }

    // VERSION
    jp = json_object_object_get(json, JA_PROTO_TAG_VERSION);
    if (jp != NULL) {
        job->version = json_object_get_int(jp);
    } else {
        zbx_snprintf(job->message, sizeof(job->message),
                     "Can't get the tag [%s] from json data [%s]",
                     JA_PROTO_TAG_VERSION, telegram);
        return 2;
    }
    if (job->version != JA_PROTO_AGENT_VERSION) {
        zbx_snprintf(job->message, sizeof(job->message),
                     "%s:[%d] does not match [%d]", JA_PROTO_TAG_VERSION,
                     job->version, JA_PROTO_AGENT_VERSION);
        return 1;
    }
    // DATA
    jp = json_object_object_get(json, JA_PROTO_TAG_DATA);
    if (jp == NULL) {
        zbx_snprintf(job->message, sizeof(job->message),
                     "Can't get the tag [%s] from json data [%s]",
                     JA_PROTO_TAG_DATA, telegram);
        return 2;
    }
    if (json_object_get_type(jp) != json_type_object) {
        zbx_snprintf(job->message, sizeof(job->message),
                     "[%s] is not an object from json data [%s]",
                     JA_PROTO_TAG_DATA, telegram);
        return 2;
    }

    return 0;
}

/******************************************************************************
 *                                                                            *
 * Function:                                                                  *
 *                                                                            *
 * Purpose:                                                                   *
 *                                                                            *
 * Parameters:                                                                *
 *                                                                            *
 * Return value:                                                              *
 *                                                                            *
 * Comments:                                                                  *
 *                                                                            *
 ******************************************************************************/
int ja_telegram_from_jobrun(const char *telegram, ja_job_object * job)
{
    int ret, chk;
    char *buf, *data;
    json_object *jp_telegram, *jp_data, *jp;
    const char *__function_name = "ja_telegram_from_jobrun";

    zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __function_name);
    if (job == NULL)
        return FAIL;

    buf = NULL;
#ifdef _WINDOWS
    buf = (char *) ja_utf8_to_acp(telegram);
#else
    buf = zbx_strdup(buf, telegram);
#endif
    zabbix_log(LOG_LEVEL_DEBUG, "In %s() telegram: %s", __function_name,
               buf);

    ret = FAIL;
    jp_telegram = json_tokener_parse(buf);
    if (is_error(jp_telegram)) {
        zbx_snprintf(job->message, sizeof(job->message),
                     "Can't parse json data. telegram [%s]", buf);
        jp_telegram = NULL;
        job->result = JA_JOBRUN_RES_FAIL;
        goto exit;
    }

    chk = ja_telegram_from_head(jp_telegram, job);
    switch (chk) {
    case 0:
        break;
    case 1:
        job->result = JA_JOBRUN_RES_VERSION_FAIL;
        goto exit;
        break;
    case 2:
        job->result = JA_JOBRUN_RES_FAIL;
        goto exit;
        break;

    }

    if (strcmp(job->kind, JA_PROTO_VALUE_JOBRUN) != 0) {
        zbx_snprintf(job->message, sizeof(job->message),
                     "the kind: %s does not match %s",
                     job->kind, JA_PROTO_VALUE_JOBRUN);
        job->result = JA_JOBRUN_RES_FAIL;
        goto exit;
    }

    jp_data = json_object_object_get(jp_telegram, JA_PROTO_TAG_DATA);
    data = (char *) json_object_get_string(jp_data);

    jp = json_object_object_get(jp_data, JA_PROTO_TAG_JOBID);
    if (jp != NULL) {
        str2uint64((char *) json_object_get_string(jp), "KMGT",
                   &(job->jobid));
    } else {
        zbx_snprintf(job->message, sizeof(job->message),
                     "Can't get the tag [%s] from json data [%s]",
                     JA_PROTO_TAG_JOBID, data);
        job->result = JA_JOBRUN_RES_FAIL;
        goto exit;
    }
    if (job->jobid <= 0) {
        zbx_snprintf(job->message, sizeof(job->message),
                     "%s:[" ZBX_FS_UI64 "] must be more than 0",
                     JA_PROTO_TAG_JOBID, job->jobid);
        job->result = JA_JOBRUN_RES_FAIL;
        goto exit;
    }

    jp = json_object_object_get(jp_data, JA_PROTO_TAG_METHOD);
    if (jp != NULL) {
        job->method = json_object_get_int(jp);
    } else {
        zbx_snprintf(job->message, sizeof(job->message),
                     "Can't get the tag [%s] from json data [%s]",
                     JA_PROTO_TAG_METHOD, data);
        job->result = JA_JOBRUN_RES_FAIL;
        goto exit;
    }

    jp = json_object_object_get(jp_data, JA_PROTO_TAG_TYPE);
    if (jp != NULL) {
        zbx_snprintf(job->type, sizeof(job->type), "%s",
                     json_object_get_string(jp));
    }

    jp = json_object_object_get(jp_data, JA_PROTO_TAG_ARGUMENT);
    if (jp != NULL) {
        zbx_snprintf(job->argument, sizeof(job->argument), "%s",
                     json_object_get_string(jp));
    }

    jp = json_object_object_get(jp_data, JA_PROTO_TAG_SCRIPT);
    if (jp != NULL) {
        zbx_snprintf(job->script, sizeof(job->script), "%s",
                     json_object_get_string(jp));
    }

    jp = json_object_object_get(jp_data, JA_PROTO_TAG_ENV);
    if (jp != NULL) {
        if (json_object_get_type(jp) != json_type_object) {
            zbx_snprintf(job->message, sizeof(job->message),
                         "[%s] is not an object from json data [%s]",
                         JA_PROTO_TAG_ENV, data);
            job->result = JA_JOBRUN_RES_FAIL;
            goto exit;
        }
        zbx_snprintf(job->env, sizeof(job->env), "%s",
                     json_object_get_string(jp));
    }

    job->result = JA_JOBRUN_RES_SUCCEED;
    ret = SUCCEED;

  exit:
    zbx_free(buf);
    if (jp_telegram != NULL)
        json_object_put(jp_telegram);
    if (ret == FAIL) {
        zabbix_log(LOG_LEVEL_ERR, "In %s() %s", __function_name,
                   job->message);
    }
    return ret;
}

/******************************************************************************
 *                                                                            *
 * Function:                                                                  *
 *                                                                            *
 * Purpose:                                                                   *
 *                                                                            *
 * Parameters:                                                                *
 *                                                                            *
 * Return value:                                                              *
 *                                                                            *
 * Comments:                                                                  *
 *                                                                            *
 ******************************************************************************/
int ja_telegram_from_jobresult_res(const char *telegram,
                                   ja_job_object * job)
{
    int ret;
    char *buf, *data;
    json_object *jp_telegram, *jp_data, *jp;
    const char *__function_name = "ja_telegram_from_jobresult_res";

    zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __function_name);
    if (job == NULL)
        return FAIL;

    buf = NULL;
#ifdef _WINDOWS
    buf = (char *) ja_utf8_to_acp(telegram);
#else
    buf = zbx_strdup(buf, telegram);
#endif
    zabbix_log(LOG_LEVEL_DEBUG, "In %s() telegram: %s", __function_name,
               buf);

    ret = FAIL;
    jp_telegram = json_tokener_parse(buf);
    if (is_error(jp_telegram)) {
        zbx_snprintf(job->message, sizeof(job->message),
                     "Can't parse json data. telegram [%s]", buf);
        jp_telegram = NULL;
        job->result = JA_JOBRESULT_RES_FAIL;
        goto exit;
    }

    if (ja_telegram_from_head(jp_telegram, job) != 0) {
        job->result = JA_JOBRESULT_RES_FAIL;
        goto exit;
    }

    if (strcmp(job->kind, JA_PROTO_VALUE_JOBRESULT_RES) != 0) {
        zbx_snprintf(job->message, sizeof(job->message),
                     "the kind: %s does not match %s",
                     job->kind, JA_PROTO_VALUE_JOBRESULT_RES);
        job->result = JA_JOBRESULT_RES_FAIL;
        goto exit;
    }

    jp_data = json_object_object_get(jp_telegram, JA_PROTO_TAG_DATA);
    data = (char *) json_object_get_string(jp_data);
    jp = json_object_object_get(jp_data, JA_PROTO_TAG_RESULT);
    if (jp != NULL) {
        job->result = json_object_get_int(jp);
    } else {
        zbx_snprintf(job->message, sizeof(job->message),
                     "Can't get the tag [%s] from json data [%s]",
                     JA_PROTO_TAG_RESULT, data);
        job->result = JA_JOBRESULT_RES_FAIL;
        goto exit;
    }

    jp = json_object_object_get(jp_data, JA_PROTO_TAG_MESSAGE);
    if (jp != NULL) {
        zbx_snprintf(job->message, sizeof(job->message), "%s",
                     json_object_get_string(jp));
    }
    ret = SUCCEED;

  exit:
    zbx_free(buf);
    if (jp_telegram != NULL)
        json_object_put(jp_telegram);
    if (ret == FAIL) {
        zabbix_log(LOG_LEVEL_ERR, "In %s() %s", __function_name,
                   job->message);
    }
    return ret;
}

/******************************************************************************
 *                                                                            *
 * Function:                                                                  *
 *                                                                            *
 * Purpose:                                                                   *
 *                                                                            *
 * Parameters:                                                                *
 *                                                                            *
 * Return value:                                                              *
 *                                                                            *
 * Comments:                                                                  *
 *                                                                            *
 ******************************************************************************/
int ja_telegram_to_head(ja_job_object * job, json_object * json)
{
    const char *__function_name = "ja_telegram_to_head";
    if (job == NULL || json == NULL)
        return FAIL;

    zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __function_name);

    json_object_object_add(json, JA_PROTO_TAG_KIND,
                           json_object_new_string(job->kind));
    json_object_object_add(json, JA_PROTO_TAG_VERSION,
                           json_object_new_int(job->version));
    json_object_object_add(json, JA_PROTO_TAG_SERVERID,
                           json_object_new_string(job->serverid));
    return SUCCEED;
}

/******************************************************************************
 *                                                                            *
 * Function:                                                                  *
 *                                                                            *
 * Purpose:                                                                   *
 *                                                                            *
 * Parameters:                                                                *
 *                                                                            *
 * Return value:                                                              *
 *                                                                            *
 * Comments:                                                                  *
 *                                                                            *
 ******************************************************************************/
int ja_telegram_to_jobresult(ja_job_object * job, char *telegram)
{
    json_object *jp_telegram, *jp_data;
    char str[JA_MAX_STRING_LEN];
#ifdef _WINDOWS
    LPSTR buf;
#endif
    const char *__function_name = "ja_telegram_to_jobresult";

    if (job == NULL)
        return FAIL;
    zabbix_log(LOG_LEVEL_DEBUG, "In %s() jobid: " ZBX_FS_UI64,
               __function_name, job->jobid);

    zbx_snprintf(job->kind, sizeof(job->kind), "%s",
                 JA_PROTO_VALUE_JOBRESULT);
    jp_telegram = json_object_new_object();
    ja_telegram_to_head(job, jp_telegram);

    jp_data = json_object_new_object();
    json_object_object_add(jp_telegram, JA_PROTO_TAG_DATA, jp_data);
    zbx_snprintf(str, sizeof(str), ZBX_FS_UI64, job->jobid);
    json_object_object_add(jp_data, JA_PROTO_TAG_JOBID,
                           json_object_new_string(str));
    json_object_object_add(jp_data, JA_PROTO_TAG_HOST,
                           json_object_new_string(CONFIG_HOSTNAME));
    json_object_object_add(jp_data, JA_PROTO_TAG_RESULT,
                           json_object_new_int(job->result));
    json_object_object_add(jp_data, JA_PROTO_TAG_METHOD,
                           json_object_new_int(job->method));
    if (job->result == JA_JOBRESULT_SUCCEED) {
        json_object_object_add(jp_data, JA_PROTO_TAG_JOBSTDOUT,
                               json_object_new_string(job->std_out));
        json_object_object_add(jp_data, JA_PROTO_TAG_JOBSTDERR,
                               json_object_new_string(job->std_err));
        json_object_object_add(jp_data, JA_PROTO_TAG_RET,
                               json_object_new_int(job->return_code));
    } else {
        json_object_object_add(jp_data, JA_PROTO_TAG_MESSAGE,
                               json_object_new_string(job->message));
    }

#ifdef _WINDOWS
    buf = ja_acp_to_utf8(json_object_to_json_string(jp_telegram));
    zbx_snprintf(telegram, JA_MAX_TELEGRAM_LEN, "%s", buf);
    zbx_free(buf);
#else
    zbx_snprintf(telegram, JA_MAX_TELEGRAM_LEN, "%s",
                 json_object_to_json_string(jp_telegram));
#endif
    json_object_put(jp_telegram);
    return SUCCEED;
}

/******************************************************************************
 *                                                                            *
 * Function:                                                                  *
 *                                                                            *
 * Purpose:                                                                   *
 *                                                                            *
 * Parameters:                                                                *
 *                                                                            *
 * Return value:                                                              *
 *                                                                            *
 * Comments:                                                                  *
 *                                                                            *
 ******************************************************************************/
int ja_telegram_to_jobrun_res(ja_job_object * job, char *telegram)
{
    json_object *jp_telegram, *jp_data;
#ifdef _WINDOWS
    LPSTR buf;
#endif
    const char *__function_name = "ja_telegram_to_jobrun_res";

    if (job == NULL)
        return FAIL;

    zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __function_name);

    zbx_snprintf(job->kind, sizeof(job->kind), "%s",
                 JA_PROTO_VALUE_JOBRUN_RES);
    jp_telegram = json_object_new_object();
    ja_telegram_to_head(job, jp_telegram);

    jp_data = json_object_new_object();
    json_object_object_add(jp_telegram, JA_PROTO_TAG_DATA, jp_data);

    json_object_object_add(jp_data, JA_PROTO_TAG_RESULT,
                           json_object_new_int(job->result));

    if (job->result != JA_JOBRUN_RES_SUCCEED) {
        json_object_object_add(jp_data, JA_PROTO_TAG_MESSAGE,
                               json_object_new_string(job->message));
    }

    if (job->result == JA_JOBRUN_RES_VERSION_FAIL) {
        json_object_object_add(jp_data, JA_PROTO_TAG_MYVERSION,
                               json_object_new_int
                               (JA_PROTO_AGENT_VERSION));
    }
#ifdef _WINDOWS
    buf = ja_acp_to_utf8(json_object_to_json_string(jp_telegram));
    zbx_snprintf(telegram, JA_MAX_TELEGRAM_LEN, "%s", buf);
    zbx_free(buf);
#else
    zbx_snprintf(telegram, JA_MAX_TELEGRAM_LEN, "%s",
                 json_object_to_json_string(jp_telegram));
#endif
    json_object_put(jp_telegram);
    return SUCCEED;
}
