/*
** Job Arranger for ZABBIX
** Copyright (C) 2012 FitechForce, Inc. All Rights Reserved.
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
**/

/*
** $Date:: 2012-12-28 11:34:40 +0900 #$
** $Revision: 3507 $
** $Author: ossinfra@FITECHLABS.CO.JP $
**/

#include "common.h"
#include "log.h"
#include "db.h"

#include "jacommon.h"
#include "javalue.h"
#include "jastatus.h"
#include "jalog.h"

/******************************************************************************
 *                                                                            *
 * Function:                                                                  *
 *                                                                            *
 * Purpose:                                                                   *
 *                                                                            *
 * Parameters:                                                                *
 *                                                                            *
 * Return value:                                                              *
 *                                                                            *
 * Comments:                                                                  *
 *                                                                            *
 ******************************************************************************/
int jajobnet_ready(const zbx_uint64_t inner_jobnet_id,
                   const zbx_uint64_t icon_jobnet_id)
{
    DB_RESULT result;
    DB_ROW row;
    zbx_uint64_t icon_start_id;
    int count;
    const char *__function_name = "jajobnet_ready";

    zabbix_log(LOG_LEVEL_DEBUG,
               "In %s() inner_jobnet_id: " ZBX_FS_UI64
               ", icon_jobnet_id: " ZBX_FS_UI64,
               __function_name, inner_jobnet_id, icon_jobnet_id);

    // search start icon
    count = 0;
    icon_start_id = 0;
    result =
        DBselect
        ("select inner_job_id from ja_run_job_table"
         " where inner_jobnet_id = " ZBX_FS_UI64 " and job_type = %d ",
         inner_jobnet_id, JA_JOB_TYPE_START);
    while (NULL != (row = DBfetch(result))) {
        count++;
        if (count == 1)
            ZBX_STR2UINT64(icon_start_id, row[0]);
        else
            icon_start_id = 0;
    }
    DBfree_result(result);

    if (ja_set_status_jobnet(inner_jobnet_id, JA_JOBNET_STATUS_RUN, 1, 0)
        == FAIL)
        return FAIL;

    if (icon_start_id == 0) {
        ja_log("JAJOBNETREADY200001", inner_jobnet_id, NULL,
               __function_name, count, inner_jobnet_id);
    } else {
        if (icon_jobnet_id != 0) {
            // set start icon env
            ja_value_after_before(icon_jobnet_id, icon_start_id,
                                  inner_jobnet_id);
        }
        if (ja_set_status_job
            (icon_start_id, JA_JOBNET_STATUS_READY, 0, 0) == FAIL)
            return FAIL;
    }
    return SUCCEED;
}
